package com.huiju.welcome.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaFirstpage;
import com.huiju.welcome.model.TaFirstpageContent;
import com.huiju.welcome.model.TaFirstpageNotice;
import com.huiju.welcome.service.ITaFirstpageContentService;
import com.huiju.welcome.service.ITaFirstpageNoticeService;
import com.huiju.welcome.service.ITaFirstpageService;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@RestController
@RequestMapping("/")
public class TaFirstpageController extends BaseController {
    @Autowired
    private ITaFirstpageService taFirstpageService;

    @Autowired
    private ITaFirstpageContentService taFirstpageContentService;

    @Autowired
    private ITaFirstpageNoticeService taFirstpageNoticeService;


    @ApiOperation(value = "获取首屏列表", notes = "获取首屏列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "data", value = "数据"),
    })
    @RequestMapping(value = "/command/firstpage", method = RequestMethod.POST)
    public ResponseBean Command(@RequestBody String data, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        JSONObject msg = JSONObject.parseObject(data);
        Integer to = (Integer) msg.get("to");
        TaFirstpage taFirstpage = taFirstpageService.getById(to);
        if (null == taFirstpage) {
            responseBean.addError(HttpStatus.SC_BAD_REQUEST, "解析发送内容失败或者没有数据");
            return responseBean;
        }

        try {
            Map<String, Object> payload = new HashMap<>();
            payload.put("to", taFirstpage.getDeviceId());
            payload.put("data", msg);
            Action act = Action.create(Action.TYPE_TO_FRONT, payload);

            WebSocketServer.sendInfo(act.toJSON(), "device", taFirstpage.getDeviceId());
        } catch (Exception e) {
            e.printStackTrace();
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "发送失败");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "获取首屏列表", notes = "获取首屏列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageNum", value = "当前页"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageSize", value = "每页数量"),
    })
    @RequestMapping(value = "/firstpage", method = RequestMethod.GET)
    public ResponseBean list(@RequestParam Integer pageNum, @RequestParam  Integer pageSize, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        Page<TaFirstpage> pg = new Page<>();
        pg.setCurrent(pageNum);
        pg.setSize(pageSize);
        QueryWrapper<TaFirstpage> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("firstpage_id");
        queryWrapper.eq("status", StatusUtils.Normal);
        responseBean.addSuccess(taFirstpageService.page(pg, queryWrapper));

        return responseBean;
    }

    @ApiOperation(value = "新增首屏内容", notes = "新增首屏内容")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "TaFirstpage", name = "taFirstpage", value = "首屏内容"),
    })
    @RequestMapping(value = "/firstpage", method = RequestMethod.POST)
    public ResponseBean create(@RequestBody TaFirstpage taFirstpage, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        taFirstpage.setStatus(StatusUtils.Normal);
        boolean success = taFirstpageService.save(taFirstpage);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "新增内容失败, 请重试");
            return responseBean;
        }

        success = saveFirstpageAttaches(taFirstpage, false);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "新增内容失败, 请重试");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "更新首屏内容", notes = "更新首屏内容")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "TaFirstpage", name = "taFirstpage", value = "首屏内容"),
    })
    @RequestMapping(value = "/firstpage", method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaFirstpage taFirstpage, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        boolean success = taFirstpageService.updateById(taFirstpage);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新内容失败, 请重试");
            return responseBean;
        }

        success = saveFirstpageAttaches(taFirstpage, true);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新内容失败, 请重试");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "删除首屏内容", notes = "删除首屏内容")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "Integer", name = "firstpageId", value = "首屏ID"),
    })
    @RequestMapping(value = "/firstpage/{firstpageId}", method = RequestMethod.DELETE)
    public ResponseBean update(@PathVariable Integer firstpageId, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        TaFirstpage taFirstpage = taFirstpageService.getById(firstpageId);
        taFirstpage.setStatus(StatusUtils.Delete);

        boolean success = taFirstpageService.updateById(taFirstpage);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "删除内容失败, 请重试");
            return responseBean;
        }

        return responseBean;
    }

    private boolean saveFirstpageAttaches(TaFirstpage taFirstpage, boolean deleteFirst) {
        Integer firstpageId = taFirstpage.getFirstpageId();
        boolean success;

        if (deleteFirst) {
            QueryWrapper<TaFirstpageContent> q1 = new QueryWrapper<>();
            q1.eq("firstpage_id", firstpageId);
            success = taFirstpageContentService.remove(q1);
            if (!success) {
                return false;
            }

            QueryWrapper<TaFirstpageNotice> q2 = new QueryWrapper<>();
            q2.eq("firstpage_id", firstpageId);
            success = taFirstpageNoticeService.remove(q2);
            if (!success) {
                return false;
            }
        }

        List<TaFirstpageContent> content = taFirstpage.getContent();
        for (TaFirstpageContent ctt : content) {
            ctt.setFirstpageId(firstpageId);
        }

        List<TaFirstpageNotice> notice = taFirstpage.getNotice();
        for (TaFirstpageNotice ctt : notice) {
            ctt.setFirstpageId(firstpageId);
        }

        success = taFirstpageContentService.saveBatch(taFirstpage.getContent());
        if (!success) {
            return false;
        }

        success = taFirstpageNoticeService.saveBatch(taFirstpage.getNotice());
        if (!success) {
            return false;
        }

        return true;
    }

    @ApiOperation(value = "获取指定首屏内容", notes = "获取指定首屏内容")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "Integer", name = "firstpageId", value = "首屏ID"),
    })
    @RequestMapping(value = "/firstpage/{firstpageId}", method = RequestMethod.GET)
    public ResponseBean get(@PathVariable int firstpageId, HttpServletRequest request) {
        return detail(firstpageId, "", request,null);
    }

    @ApiOperation(value = "前端获取指定首屏内容", notes = "前端获取指定首屏内容")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "String", name = "deviceId", value = "设备ID"),
            @ApiImplicitParam(paramType = "Query", dataType = "String", name = "city", value = "地址")
    })
    @RequestMapping(value = "/screen/fristpage/device/{deviceId}", method = RequestMethod.GET)
    public ResponseBean screenGet(@PathVariable(value = "deviceId") String deviceId, HttpServletRequest request,
                                  @RequestParam(value ="city", required = false) String city) {
        return detail(0, deviceId, request, city);
    }

    // getDetail 获取首屏详情
    private ResponseBean detail(int firstpageId, String deviceId, HttpServletRequest request,String city) {
        ResponseBean responseBean = new ResponseBean();

        QueryWrapper<TaFirstpage> q = new QueryWrapper<>();
        if (firstpageId > 0) {
            q.eq("firstpage_id", firstpageId);
        } else {
            q.eq("device_id", deviceId);
        }

        TaFirstpage taFirstpage = taFirstpageService.getOne(q);
        if (null == taFirstpage) {
            responseBean.addError(HttpStatus.SC_NOT_FOUND, "内容不存在");
            return responseBean;
        }
        //
        taFirstpage.setCity(city);
        // 获取内容
        Map<String, Object> ctt = taFirstpageService.detail(taFirstpage);
        responseBean.addSuccess(ctt);
        return responseBean;
    }
}
