package com.huiju.welcome.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TdSpec;
import com.huiju.welcome.service.ITdSpecService;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-07-16
 */
@RestController
@RequestMapping("/")
public class TdSpecController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TdSpecController.class);

    @Autowired
    public ITdSpecService iTdSpecService;


    /**
     * 保存对象
     * @param tdSpec 实体对象
     * @return
     */
    @RequestMapping(value="/tdSpec/add",method= RequestMethod.POST)
    public ResponseBean tpShopImgAdd(@RequestBody TdSpec tdSpec){
        ResponseBean responseBean = new ResponseBean();
        try {
            tdSpec.setCreateDate(LocalDateTime.now());
            tdSpec.setStatus(StatusUtils.Normal);
            if (iTdSpecService.save(tdSpec)){
                responseBean.addSuccess(tdSpec);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("tdSpecAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/tdSpec/delete/{id}", method= RequestMethod.PUT)
    public ResponseBean tdSpecDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            TdSpec tdSpec = iTdSpecService.getById(id);
            tdSpec.setStatus(StatusUtils.Delete);
            if(iTdSpecService.updateById(tdSpec)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("tdSpecDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param tdSpec 实体对象
     * @return
     */
    @RequestMapping(value="/tdSpec/update",method= RequestMethod.PUT)
    public ResponseBean tpShopImgUpdate(@RequestBody TdSpec tdSpec){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTdSpecService.updateById(tdSpec)){
                responseBean.addSuccess(tdSpec);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("tdSpecUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/tdSpec/get/{id}",method= RequestMethod.GET)
    public ResponseBean tpShopImgGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTdSpecService.getById(id));
        }catch (Exception e){
            logger.error("tdSpecDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @ApiOperation(value = "商品规格列表", notes = "商品规格列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "specName", value = "商品规格名称")
    })
    @RequestMapping(value = "/tdSpec", method = RequestMethod.GET)
    public ResponseBean specList(@RequestParam(defaultValue = "1") int pageNum,
                                 @RequestParam(defaultValue = "10") int pageSize,
                                 @RequestParam(value = "specName",required = false) String specName) {
        IPage<TdSpec> pg = new Page<>(pageNum, pageSize);

        ResponseBean userManagement= iTdSpecService.specList(pg,specName);

        return userManagement;
    }
}
